'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.Resource = exports.resources = exports.Client = undefined;

var _get = function get(object, property, receiver) { if (object === null) object = Function.prototype; var desc = Object.getOwnPropertyDescriptor(object, property); if (desc === undefined) { var parent = Object.getPrototypeOf(object); if (parent === null) { return undefined; } else { return get(parent, property, receiver); } } else if ("value" in desc) { return desc.value; } else { var getter = desc.get; if (getter === undefined) { return undefined; } return getter.call(receiver); } };

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _querystring = require('./querystring');

var _querystring2 = _interopRequireDefault(_querystring);

var _lodash = require('lodash');

var _exceptions = require('./exceptions');

var _xml = require('./xml');

var _lang = require('./lang');

var _models = require('./models');

var _models2 = _interopRequireDefault(_models);

var _sort = require('./sort');

var _sort2 = _interopRequireDefault(_sort);

var _lru = require('./lru');

var _lru2 = _interopRequireDefault(_lru);

var _string = require('./string');

var _string2 = _interopRequireDefault(_string);

var _nodeFetch = require('node-fetch');

var _nodeFetch2 = _interopRequireDefault(_nodeFetch);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var integer = _lang.coerce.integer;

var P = Promise;
var noop = function noop() {};
var dummylogger = { log: noop, info: noop };

////////////////////////////////////////////////////////////////////////////////

/**
 * HTTP client
 */
var Client = exports.Client = function () {
  _createClass(Client, [{
    key: 'defaults',


    /**
     * Return instance configuration defaults
     * @param void
     * @return {Object}
     */
    value: function defaults() {
      var location = global.location || {
        protocol: 'https:',
        host: 'localhost'
      };

      return {
        language: 'en',

        // these must match your PrestaShop backend languages
        languages: {
          // ISO code => PrestaShop language id
          'en': 1
        },

        // PrestaShop web service parameters
        webservice: {
          key: 'your-prestashop-key',
          scheme: 'https',
          host: 'your-prestashop-host',
          root: '/api'
        },

        // LRUCache instance; see https://www.npmjs.com/package/lru-cache
        cache: _lru2.default.instance(),

        // logger
        logger: dummylogger,

        // Fetch-related options
        fetch: {

          // the actual fetch function; facilitates testing
          algo: _nodeFetch2.default
        }
      };
    }

    /**
     * @param {Object} options
     */

  }]);

  function Client() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Client);

    this.options = (0, _lodash.merge)(this.defaults(), options);
    this.fetch = this.options.fetch.algo;
    this.cache = this.options.cache;
    this.logger = this.options.logger;
    this.funnel = {};
  }

  /**
   * Return a Prestashop API language id
   * @property
   * @type {Number}
   */


  _createClass(Client, [{
    key: 'get',


    /**
     * Send a GET request
     * @async Promise
     * @param {String} url
     * @param {Object} query
     * @return {Response}
     */
    value: function get(uri) {
      var _this = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var url = this.url(uri, options.query);
      var key = this.options.language + ':GET:' + url;
      var response = this.cache.get(key);
      var funnel = this.funnel;

      if (response) {
        return P.resolve(response);
      }

      // multiple requests on the same key converge on a single promise
      if (funnel[key]) {
        return funnel[key];
      }

      var fopts = this.createFetchOptions(_extends({}, options.fetch, { method: 'GET' }));

      funnel[key] = this.fetch(url, fopts).then(function (response) {
        delete funnel[key];
        _this.validateResponse(response);
        _this.cache.set(key, response);
        return response;
      }).catch(function (e) {
        delete funnel[key];
        throw e;
      });

      return funnel[key];
    }

    /**
     * Return a fully qualified API url
     * @param {String} uri
     * @param {String}
     */

  }, {
    key: 'url',
    value: function url(uri) {
      var query = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var qs = '';

      if (!(0, _lang.empty)(query)) {
        query = (0, _lang.tuples)(query).sort();
        qs = '?' + _querystring2.default.stringify(query);
      }

      if (uri.indexOf('http') === 0) {
        return uri + qs;
      }

      var webservice = this.options.webservice;

      var fullpath = _path2.default.join(webservice.root, uri);

      return webservice.scheme + '://' + webservice.host + fullpath + qs;
    }

    /**
     * Return node-fetch request options
     * @param {Object} augments
     * @return {Object}
     */

  }, {
    key: 'createFetchOptions',
    value: function createFetchOptions() {
      var augments = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      return _extends({}, this.options.fetch.defaults, {
        headers: this.createHeaders()
      }, augments);
    }

    /**
     * Return a dictionary containing request headers
     * @param {Object} augments
     * @return {Object}
     */

  }, {
    key: 'createHeaders',
    value: function createHeaders() {
      var augments = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      var _options$webservice = _extends({}, this.options.webservice);

      var key = _options$webservice.key;

      var Authorization = this.createAuthorizationHeader(key);

      return _extends({}, augments, { Authorization: Authorization });
    }

    /**
     * @param {Response} response
     * @return void
     * @throws Error
     */

  }, {
    key: 'validateResponse',
    value: function validateResponse(response) {
      if (!response.ok) {
        throw new _exceptions.UnexpectedValue('got non-2XX HTTP response');
      }
    }

    /**
     * Return an Authorization header value given a web service key
     * @param {String} key
     * @return {String}
     */

  }, {
    key: 'createAuthorizationHeader',
    value: function createAuthorizationHeader(key) {
      var username = key;
      var password = '';
      var precursor = username + ':' + password;
      var credentials = Buffer.from(precursor).toString('base64');

      return 'Basic ' + credentials;
    }

    /**
     * @param {String} api - snake case form of a resource class name
     * @param {Object} options
     * @return {rest.Resource}
     */

  }, {
    key: 'resource',
    value: function resource(api) {
      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var classname = _string2.default.studly(api);
      var constructor = resources[classname];

      if (!constructor) {
        throw new _exceptions.InvalidArgument('invalid root resource: "' + api + '"');
      }

      return new constructor(_extends({}, options, {
        client: this,
        logger: this.logger
      }));
    }
  }, {
    key: 'language',
    get: function get() {
      var _options = this.options;
      var languages = _options.languages;
      var isocode = _options.language;

      return languages[isocode];
    }
  }]);

  return Client;
}();

////////////////////////////////////////////////////////////////////////////////
var resources = exports.resources = {};

/**
 * Resource is an HTTP-aware context that uses a Client to fetch XML payloads
 * from the PrestaShop API. It converts XML payloads into Model instances.
 */
var Resource = exports.Resource = resources.Resource = function () {
  _createClass(_class, [{
    key: 'defaults',


    /**
     * Return instance configuration defaults
     * @param void
     * @return {Object}
     */
    value: function defaults() {
      var classname = this.constructor.name;
      var api = _string2.default.snake(classname);
      var nodetype = api.slice(0, -1);
      var root = '/' + api;
      var modelname = classname.slice(0, -1);

      return {
        client: null,
        logger: dummylogger,
        model: _models2.default[modelname],
        root: root,
        api: api,
        nodetype: nodetype,

        // model list filter function
        filter: null,

        // model list sort function
        sort: null
      };
    }

    /**
     * @param {Object} options
     */

  }], [{
    key: 'name',


    /**
     * Return the canonical constructor name. This is a workaround to a side
     * effect of Babel transpilation, which is that Babel mangles class names.
     * @property
     * @type {String}
     */
    get: function get() {
      return 'Resource';
    }
  }]);

  function _class() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, _class);

    this.options = _extends({}, this.defaults(), options);
    this.client = this.options.client;
    this.logger = this.options.logger;
  }

  /**
   * Return a PrestaShop language id
   * @property
   * @type {Number}
   */


  _createClass(_class, [{
    key: 'list',


    /**
     * Resolve an array of Model instances
     * @async Promise
     * @param void
     * @return {Array}
     */
    value: function list() {
      var _this2 = this;

      return this.client.get(this.options.root).then(function (response) {
        return response.clone().text();
      }).then(function (xml) {
        return _this2.parseModelIds(xml);
      }).then(function (ids) {
        return _this2.createModels(ids);
      }).then(function (models) {
        var _options2 = _this2.options;
        var sort = _options2.sort;
        var filter = _options2.filter;


        if (filter) {
          models = models.filter(filter);
        }
        if (sort) {
          models = models.sort(sort);
        }

        return models;
      });
    }

    /**
     * Return the first member of a list() result
     * @param void
     * @return {models.Model}
     */

  }, {
    key: 'first',
    value: function first() {
      return this.list().then(function (models) {
        return models.shift() || null;
      });
    }

    /**
     * Resolve a single Model instance, or null if there was an error of any kind
     * @async Promise
     * @param {mixed} id
     * @return {Model|null}
     */

  }, {
    key: 'get',
    value: function get(id) {
      var _this3 = this;

      var uri = this.options.root + '/' + id;
      var promise = void 0;

      try {
        promise = this.client.get(uri);
      } catch (e) {
        this.logger.log('failed to acquire model properties on request path ' + uri);
        this.logger.log(e.message);
        this.logger.log(e.stack);

        // FIXME
        return P.resolve(this.createModel());
      }

      return promise.then(function (response) {
        return response.clone().text();
      }).then(function (xml) {
        return _this3.parseModelAttributes(xml);
      }).then(function (attrs) {
        return _this3.createModel(attrs);
      });
    }

    /**
     * Map a list of model ids to model instances.
     * @async Promise
     * @param {Array} ids
     * @return {Array}
     */

  }, {
    key: 'createModels',
    value: function createModels(ids) {
      var _this4 = this;

      return P.all(ids.map(function (id) {
        return _this4.get(id);
      }));
    }

    /**
     * Given an object containing model properties, return a Model instance.
     * @param {Object} attrs
     * @return {Model}
     */

  }, {
    key: 'createModel',
    value: function createModel() {
      var attrs = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var constructor = this.options.model;


      return new constructor({
        client: this.client,
        resource: this,
        attrs: attrs
      });
    }

    /**
     * Given the API response payload for a list of objects, return a list of
     * model ids.
     * @async Promise
     * @param {String} xml
     * @return {Array}
     */

  }, {
    key: 'parseModelIds',
    value: function parseModelIds(xml) {
      var _options3 = this.options;
      var api = _options3.api;
      var nodetype = _options3.nodetype;

      return _xml.parse.model.ids(xml, api, nodetype);
    }

    /**
     * Given the API response payload for a single domain object, return a plain
     * object that contains model properties.
     * @param {String} xml
     * @return {Object}
     */

  }, {
    key: 'parseModelAttributes',
    value: function parseModelAttributes(xml) {
      var nodetype = this.options.nodetype;
      var ns = _xml.parse[nodetype];

      if (!ns) {
        throw new _exceptions.UnexpectedValue('parser namespace not found on node type ' + nodetype);
      }
      if (!ns.attributes) {
        throw new _exceptions.UnexpectedValue('model properties parser not found on node type ' + nodetype);
      }

      return _xml.parse[nodetype].attributes(xml, this.language);
    }
  }, {
    key: 'language',
    get: function get() {
      return this.client.language;
    }
  }]);

  return _class;
}();

resources.Products = function (_Resource) {
  _inherits(_class2, _Resource);

  function _class2() {
    _classCallCheck(this, _class2);

    return _possibleConstructorReturn(this, (_class2.__proto__ || Object.getPrototypeOf(_class2)).apply(this, arguments));
  }

  _createClass(_class2, null, [{
    key: 'name',


    /**
     * inheritdoc
     */
    get: function get() {
      return 'Products';
    }
  }]);

  return _class2;
}(Resource);

resources.Images = function (_Resource2) {
  _inherits(_class3, _Resource2);

  function _class3() {
    _classCallCheck(this, _class3);

    return _possibleConstructorReturn(this, (_class3.__proto__ || Object.getPrototypeOf(_class3)).apply(this, arguments));
  }

  _createClass(_class3, [{
    key: 'list',


    /**
     * @inheritdoc
     */
    value: function list() {
      var _this7 = this;

      return this.client.get(this.options.root).then(function (response) {
        return response.clone().text();
      }).then(function (xml) {
        return _this7.parseImageAttributes(xml);
      }).then(function (attrsets) {
        return attrsets.map(function (attrs) {
          return _this7.createModel(attrs);
        });
      });
    }

    /**
     * Return a list of urls given an XML payload
     * @async Promise
     * @param {String} xml
     * @return {Array}
     */

  }, {
    key: 'parseImageAttributes',
    value: function parseImageAttributes(xml) {
      return _xml.parse.image.attributes(xml);
    }
  }], [{
    key: 'name',
    get: function get() {
      return 'Images';
    }
  }]);

  return _class3;
}(Resource);

resources.Manufacturers = function (_Resource3) {
  _inherits(_class4, _Resource3);

  function _class4() {
    _classCallCheck(this, _class4);

    return _possibleConstructorReturn(this, (_class4.__proto__ || Object.getPrototypeOf(_class4)).apply(this, arguments));
  }

  _createClass(_class4, null, [{
    key: 'name',


    /**
     * @inheritdoc
     */
    get: function get() {
      return 'Manufacturers';
    }
  }]);

  return _class4;
}(Resource);

resources.Combinations = function (_Resource4) {
  _inherits(_class5, _Resource4);

  function _class5() {
    _classCallCheck(this, _class5);

    return _possibleConstructorReturn(this, (_class5.__proto__ || Object.getPrototypeOf(_class5)).apply(this, arguments));
  }

  _createClass(_class5, null, [{
    key: 'name',


    /**
     * @inheritdoc
     */
    get: function get() {
      return 'Combinations';
    }
  }]);

  return _class5;
}(Resource);

resources.StockAvailables = function (_Resource5) {
  _inherits(_class6, _Resource5);

  function _class6() {
    _classCallCheck(this, _class6);

    return _possibleConstructorReturn(this, (_class6.__proto__ || Object.getPrototypeOf(_class6)).apply(this, arguments));
  }

  _createClass(_class6, null, [{
    key: 'name',


    /**
     * @inheritdoc
     */
    get: function get() {
      return 'StockAvailables';
    }
  }]);

  return _class6;
}(Resource);

resources.ProductOptionValues = function (_Resource6) {
  _inherits(_class7, _Resource6);

  function _class7() {
    _classCallCheck(this, _class7);

    return _possibleConstructorReturn(this, (_class7.__proto__ || Object.getPrototypeOf(_class7)).apply(this, arguments));
  }

  _createClass(_class7, [{
    key: 'defaults',


    /**
     * @inheritdoc
     */
    value: function defaults() {
      return _extends({}, _get(_class7.prototype.__proto__ || Object.getPrototypeOf(_class7.prototype), 'defaults', this).call(this), {
        sort: _sort2.default.ascending(function (model) {
          return model.position;
        })
      });
    }
  }], [{
    key: 'name',


    /**
     * inheritdoc
     */
    get: function get() {
      return 'ProductOptionValues';
    }
  }]);

  return _class7;
}(Resource);

exports.default = { Client: Client, resources: resources };