'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.parse = undefined;

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol ? "symbol" : typeof obj; };

var _xml2js = require('xml2js');

var _xml2js2 = _interopRequireDefault(_xml2js);

var _lang = require('./lang');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var bool = _lang.coerce.bool;
var number = _lang.coerce.number;
var integer = _lang.coerce.integer;
var string = _lang.coerce.string;

var P = Promise;

////////////////////////////////////////////////////////////////////////////////
var xml = {};

exports.default = xml;

/**
 * Return a plain object representation of the supplied XML payload
 * @async Promise
 * @param {String|Buffer} xml
 * @return {Object}
 */

var parse = exports.parse = xml.parse = function (xml) {
  return new P(function (resolve, reject) {
    _xml2js2.default.parseString(String(xml), function (err, result) {
      err ? reject(err) : resolve(result);
    });
  });
};

/**
 * Extract the text value from a node 
 * @param mixed input
 * @return {String}
 */
var text = function text(input) {
  var raw = Array.isArray(input) ? input[0] : input;

  if ((typeof raw === 'undefined' ? 'undefined' : _typeof(raw)) == 'object') {
    raw = raw._ || '';
  }

  return String(raw).trim();
};

/**
 * Extract a node attribute value
 * @param {mixed} input
 * @return {String}
 */
var attr = function attr(input, name) {
  var raw = input;

  if (Array.isArray(raw)) {
    raw = raw[0];
  }

  if (raw.$ && raw.$[name] !== undefined) {
    raw = raw.$[name];
  }

  return String(raw).trim();
};

////////////////////////////////////////////////////////////////////////////////

parse.model = {};

/**
 * Parse the supplied XML payload and return an array of model ids.
 * @async Promise
 * @param {String|Buffer} xml
 * @param {String} api
 * @param {String} nodetype
 * @return {Array} 
 */
parse.model.ids = function (xml, api, nodetype) {
  return parse(xml).then(function (obj) {
    var list = obj.prestashop[api][0][nodetype];
    return list.map(function (obj) {
      return integer(attr(obj, 'id'));
    });
  });
};

////////////////////////////////////////////////////////////////////////////////

parse.product = {};

/**
 * @async Promise
 * @param {String} xml
 * @return {Object}
 */
parse.product.attributes = function (xml) {
  var language = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

  return parse(xml).then(function (obj) {
    var base = obj.prestashop.product[0];
    var assocs = base.associations[0] || {};
    var lang = function lang(obj) {
      return attr(obj, 'id') == language;
    };

    var names = base.name[0].language;
    var descs = base.description[0].language;
    var shortdescs = base.description_short[0].language;
    var combos = assocs.combinations[0].combination || [];
    var images = assocs.images[0].image || [];;

    return {
      'id': text(base.id),
      'name': text(names.filter(lang).pop()),
      'description': text(descs.filter(lang).pop()),
      'description_short': text(shortdescs.filter(lang).pop()),
      'price': text(base.price),
      'available_for_order': text(base.available_for_order),
      'related': {
        'manufacturer': integer(text(base.id_manufacturer)),
        'combinations': combos.map(function (combo) {
          return integer(text(combo.id));
        }),
        'images': images.map(function (image) {
          return integer(text(image.id));
        })
      }
    };
  });
};

////////////////////////////////////////////////////////////////////////////////

parse.combination = {};

/**
 * @async Promise
 * @param {String} xml
 * @return {Object}
 */
parse.combination.attributes = function (xml) {
  return parse(xml).then(function (obj) {
    var base = obj.prestashop.combination[0];
    var assocs = base.associations[0] || {};
    var povs = assocs.product_option_values || [];

    return {
      'id': text(base.id),
      'id_product': text(base.id_product),
      'location': text(base.location),
      'ean13': text(base.ean13),
      'upc': text(base.upc),
      'quantity': text(base.quantity),
      'reference': text(base.reference),
      'supplier_reference': text(base.supplier_reference),
      'wholesale_price': text(base.wholesale_price),
      'price': text(base.price),
      'ecotax': text(base.ecotax),
      'weight': text(base.weight),
      'unit_price_impact': text(base.unit_price_impact),
      'minimal_quantity': text(base.minimal_quantity),
      'default_on': text(base.default_on),
      'available_date': text(base.available_date),
      'related': {
        'product': integer(text(base.id_product)),
        'product_option_values': integer(text(povs[0].product_option_value[0].id))
      }
    };
  });
};

////////////////////////////////////////////////////////////////////////////////

parse.image = {};

/**
 * @async Promise
 * @param {String} xml
 * @return {Object}
 */
parse.image.attributes = function (xml) {
  return parse(xml).then(function (obj) {
    var decs = obj.prestashop.image[0].declination;

    return decs.map(function (dec) {
      return {
        'id': dec.$.id,
        'src': dec.$['xlink:href']
      };
    });
  });
};

////////////////////////////////////////////////////////////////////////////////

parse.manufacturer = {};

parse.manufacturer.attributes = function (xml) {
  var language = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

  return parse(xml).then(function (obj) {
    var base = obj.prestashop.manufacturer[0];
    var lang = function lang(obj) {
      return attr(obj, 'id') == language;
    };
    var descs = base.description[0].language;

    return {
      'id': text(base.id),
      'active': text(base.active),
      'name': text(base.name),
      'description': text(descs.filter(lang).pop())
    };
  });
};

////////////////////////////////////////////////////////////////////////////////

parse.stock_available = {};

parse.stock_available.attributes = function (xml) {
  var language = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

  return parse(xml).then(function (obj) {
    var base = obj.prestashop.stock_available[0];

    return {
      'id': text(base.id),
      'id_product': text(base.id_product),
      'id_product_attribute': text(base.id_product_attribute),
      'id_shop': text(base.id_shop),
      'id_shop_group': text(base.id_shop_group),
      'quantity': text(base.quantity),
      'depends_on_stock': text(base.depends_on_stock),
      'out_of_stock': text(base.out_of_stock)
    };
  });
};

////////////////////////////////////////////////////////////////////////////////

parse.product_option_value = {};

parse.product_option_value.attributes = function (xml) {
  var language = arguments.length <= 1 || arguments[1] === undefined ? 1 : arguments[1];

  return parse(xml).then(function (obj) {
    var base = obj.prestashop.product_option_value[0];
    var names = base.name[0].language;
    var lang = function lang(obj) {
      return attr(obj, 'id') == language;
    };

    return {
      'id': text(base.id),
      'id_attribute_group': text(base.id_attribute_group),
      'color': text(base.color),
      'position': text(base.position),
      'name': text(names.filter(lang).pop())
    };
  });
};